/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.model;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a domin object field
 *
 * @author shaines
 */
public class Field
{
    /**
     * The name of the field
     */
    private String name;

    /**
     * The field's type
     */
    private String type;

    /**
     * Is the field a primary key for the object?
     */
    private boolean primaryKey;

    /**
     * The list of stereotypes that this field implements
     */
    private List<Stereotype> stereotypes = new ArrayList<Stereotype>();

    /**
     * Creates a new field object
     */
    public Field()
    {
    }

    /**
     * Creates a new field object
     *
     * @param name              The name of the field
     */
    public Field( String name )
    {
        this.name = name;
    }

    /**
     * Creates a new field object
     *
     * @param name              The name of the field
     * @param type              The type of the field
     * @param primaryKey        Is the field a primary key?
     */
    public Field( String name, String type, boolean primaryKey )
    {
        this.name = name;
        this.type = type;
        this.primaryKey = primaryKey;
    }

    /**
     * Creates a new field object, defaulting the primary key identifier to false
     *
     * @param name              The name of the field
     * @param type              The type of the field
     */
    public Field( String name, String type )
    {
        this.name = name;
        this.type = type;
        this.primaryKey = false;
    }

    /**
     * Returns the name of the field
     *
     * @return                  The name of the field
     */
    public String getName()
    {
        return name;
    }

    /**
     * Sets the name of the field
     *
     * @param name              The name of the field
     */
    public void setName( String name )
    {
        this.name = name;
    }

    /**
     * Returns true if this field is a primary key, otherwise false
     *
     * @return                  True if this field is a primary key, otherwise false
     */
    public boolean isPrimaryKey()
    {
        return primaryKey;
    }

    /**
     * Sets the primary key indicator for this object: true = primary key, false = not a primary key
     *
     * @param primaryKey        True if this field is a primary key, false if it is not
     */
    public void setPrimaryKey( boolean primaryKey )
    {
        this.primaryKey = primaryKey;
    }

    /**
     * Returns field's type
     *
     * @return                  The field's type
     */
    public String getType()
    {
        return type;
    }

    /**
     * Sets the field's type
     *
     * @param type              The field's type
     */
    public void setType( String type )
    {
        this.type = type;
    }

    /**
     * Returns the list of stereotypes that this field implements
     *
     * @return                  The list of stereotypes that this field implements
     */
    public List<Stereotype> getStereotypes()
    {
        return stereotypes;
    }

    /**
     * Returns true if the Field contains the named stereotype
     *
     * @param name              The name of the stereotype to check for
     *
     * @return                  True if the field contains the stereotype, else false
     */
    public boolean containsStereotype( String name )
    {
        for( Stereotype stereotype : stereotypes )
        {
            if( stereotype.getName().equalsIgnoreCase( name ) )
            {
                return true;
            }
        }
        return false;
    }
}
