/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.model;

/**
 * Represents a model stereotype. Class stereotypes will have a package name associtated
 * with them while attribute and association end stereotypes do not have an associated
 * package name.
 *
 * @author shaines
 */
public class Stereotype implements Comparable<Stereotype>
{
    /**
     * The available types of stereotypes
     */
    public enum Type
    {
        CLASS, ATTRIBUTE, ASSOCIATION_END

    }
    /**
     * The type of this stereotype
     */
    private Type type;

    /**
     * The name of the stereotype
     */
    private String name;

    /**
     * The name of the package in which this stereotype resides; only valid for class-level
     * stereotypes
     */
    private String packageName;

    /**
     * Creates a new stereotype
     */
    public Stereotype()
    {
    }

    public Stereotype( String name, Type type )
    {
        this.name = name;
        this.type = type;
    }

    public Stereotype( String packageName, String name, Type type )
    {
        this.packageName = packageName;
        this.name = name;
        this.type = type;
    }

    public String getName()
    {
        return name;
    }

    public void setName( String name )
    {
        this.name = name;
    }

    public String getPackageName()
    {
        return packageName;
    }

    public void setPackageName( String packageName )
    {
        this.packageName = packageName;
    }

    public Type getType()
    {
        return type;
    }

    public void setType( Type type )
    {
        this.type = type;
    }

    public String toString()
    {
        if( type == Type.CLASS )
        {
            return packageName + "." + name + " (" + type + ")";
        }
        return name + " (" + type + ")";
    }

    /**
     * Sorts stereotypes by their names
     *
     * @param o             The Stereotype to which to compare this stereotype
     *
     * @return              -1, 1, or 0, depending on whether the other Stereotype is
     *                      less than, greater than, or equal to this one
     */
    @Override
    public int compareTo( Stereotype o )
    {
        if( type == Type.CLASS )
        {
            String fullyQualifiedName = packageName + "." + name;
            String otherFullyQualifiedName = o.getPackageName() + "." + o.getName();
            return fullyQualifiedName.compareTo( otherFullyQualifiedName );
        }
        return name.compareTo( o.getName() );
    }
}
