/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.service;

import com.geekcap.modeler.dao.DomainObjectDao;
import com.geekcap.modeler.model.DomainModel;
import com.geekcap.modeler.model.DomainObject;
import com.geekcap.modeler.model.Operation;
import com.geekcap.modeler.model.OperationParameter;
import com.geekcap.modeler.model.Stereotype;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.StringTokenizer;

/**
 *
 * @author shaines
 */
public class ModelerServiceImpl implements ModelerService
{
    /**
     * Used to perform the actual transformation of a domain model into the requested artifact
     */
    private TemplateService templateService;

    /**
     * The DAO to use to load the domain objects
     */
    private DomainObjectDao domainObjectDao;

    public ModelerServiceImpl()
    {
    }

    public ModelerServiceImpl( TemplateService templateService, DomainObjectDao domainObjectDao )
    {
        this.templateService = templateService;
        this.domainObjectDao = domainObjectDao;
    }

    /**
     * Executes the appropriate template to convert the source reader into the destination writer.
     *
     * @param source        An input stream that references the source UML model
     * @param destination   The destination location to which to generate the artifacts
     * @param artifact      The type of artifact to generate
     */
    public void generate( InputStream source, String destination, String artifact )
    {
        try
        {
            // Load the domain objects
            //List<DomainObject> domainObjects = domainObjectDao.loadDomainObjects( source );
            DomainModel domainModel = domainObjectDao.loadDomainModel( source );



            // TODO: For now just generate Hibernate classes, need a plan for generating other artifacts
            //for( DomainObject domainObject : domainModel.getDomainObjects() )
            for( DomainObject domainObject : domainModel.findDomainObjectsByStereotype( "Entity" ) )
            {
                System.out.print( domainObject.getName() + " Stereotypes: " );
                for( Stereotype stereotype : domainObject.getStereotypes() )
                {
                    System.out.print( stereotype + ", " );
                }
                System.out.println();

                // Generate the Hibernate class for the object
                String generatedClass = templateService.transform( domainObject, "hibernate.vm" );

                // Save it to the requested directory
                PrintStream out = new PrintStream( new FileOutputStream( getDestinationJavaFilename( destination, domainObject ) ) );
                out.print( generatedClass );
                out.close();
            }


            // Handle repository classes
            System.out.println( "Repository classes: " );
            for( DomainObject domainObject : domainModel.findDomainObjectsByStereotype( "Repository" ) )
            {
                System.out.println( "\t" + domainObject.getName() );
                System.out.println( "\t\tOperations: " );
                for( Operation operation : domainObject.getOperations() )
                {
                    System.out.println( "\t\t\t" + operation.getVisibility() + " "
                            + operation.getReturnType() + " "
                            + operation.getName() + ", fields: " );
                    for( OperationParameter parameter : operation.getParameters() )
                    {
                        System.out.println( "\t\t\t\t" + parameter );
                    }
                }

                // Build the DAO inteface
                String generatedClass = templateService.transform( domainObject, "hibernate-daointerface.vm" );

                // Save it to the requested directory
                PrintStream out = new PrintStream( new FileOutputStream( getDestinationJavaFilename( destination, domainObject ) ) );
                out.print( generatedClass );
                out.close();

                // Build the DAO implementation
                generatedClass = templateService.transform( domainObject, "hibernate-daoimpl.vm" );
                out = new PrintStream( new FileOutputStream( getDestinationJavaFilename( destination, domainObject, "Impl" ) ) );
                out.print( generatedClass );
                out.close();
            }
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
    }

    protected String getDestinationJavaFilename( String path, DomainObject domainObject )
    {
        return getDestinationJavaFilename( path, domainObject, null );
    }
    
    protected String getDestinationJavaFilename( String path, DomainObject domainObject, String nameSuffix )
    {
        // Initialize the StringBuilder with the path
        StringBuilder sb = new StringBuilder( path );
        if(  ! path.endsWith( File.separator ) )
        {
            sb.append( File.separator );
        }

        // Build the package
        StringTokenizer st = new StringTokenizer( domainObject.getPackageName(), ".", false );
        while( st.hasMoreTokens() )
        {
            sb.append( st.nextToken() );
            sb.append( File.separator );
        }

        // Create the path (if we need to)
        createPath( sb.toString() );

        // Add the class name
        sb.append( domainObject.getName() );

        // Allows us to suffix a Java class name, such as with "Impl"
        if( nameSuffix != null )
        {
            sb.append( nameSuffix );
        }

        sb.append( ".java" );

        // Return the filename we built
        return sb.toString();
    }

    protected void createPath( String path )
    {
        StringTokenizer st = new StringTokenizer( path, File.separator, false );
        StringBuilder sb = new StringBuilder();
        while( st.hasMoreTokens() )
        {
            sb.append( st.nextToken() );
            File f = new File( sb.toString() );
            if(  ! f.exists() )
            {
                if(  ! f.mkdir() )
                {
                    System.out.println( "An error occurred while creating directory: " + sb );
                    return;
                }
            }
            sb.append( File.separator );
        }
    }

    public DomainObjectDao getDomainObjectDao()
    {
        return domainObjectDao;
    }

    public void setDomainObjectDao( DomainObjectDao domainObjectDao )
    {
        this.domainObjectDao = domainObjectDao;
    }

    public TemplateService getTemplateService()
    {
        return templateService;
    }

    public void setTemplateService( TemplateService templateService )
    {
        this.templateService = templateService;
    }
}
