/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.service;

import com.geekcap.modeler.model.DomainModel;
import com.geekcap.modeler.model.DomainObject;
import com.geekcap.modeler.util.ConversionUtil;
import com.geekcap.modeler.util.OperationUtils;
import com.geekcap.modeler.util.RepositoryUtils;
import java.io.StringWriter;
import java.util.Properties;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

/**
 * TemplateService implemented by Velocity
 *
 * @author shaines
 */
public class VelocityTemplateServiceImpl extends AbstractTemplateServiceImpl
{
    /**
     * The VelocityEngine that performs our actual code generation
     */
    private VelocityEngine velocityEngine;

    public VelocityTemplateServiceImpl()
    {
        try
        {
            // Create the velocity engine with the velocity.properties file in the CLASSPATH
            Properties properties = new Properties();
            properties.load( getClass().getClassLoader().getResourceAsStream( "velocity.properties" ) );
            velocityEngine = new VelocityEngine( properties );
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
    }

    /**
     * Transforms the domain object using the specified template and returns the result
     *
     * @param domainObject          The domain object to transform
     * @param templateName          The name of the template that should be used to perform the transformation
     *
     * @return                      The transformed object
     */
    @Override
    public String transform( DomainObject domainObject, String templateName )
    {
        try
        {
            // Load the template
            //String template = templateDao.loadTemplate( templateName );
            Template template = velocityEngine.getTemplate( templateName );

            // Setup the VelocityContext
            ConversionUtil conversionUtil = new ConversionUtil();
            VelocityContext velocityContext = new VelocityContext();
            velocityContext.put( "domainObject", domainObject );
            velocityContext.put( "conversionUtil", conversionUtil );
            velocityContext.put( "operationUtils", new OperationUtils() );
            velocityContext.put( "repositoryUtils", new RepositoryUtils() );

            // Execute the template
            StringWriter writer = new StringWriter();
            velocityEngine.mergeTemplate( templateName, "utf-8", velocityContext, writer );

            // Return the result
            return writer.toString();
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * Transforms the list of domain objects using the specified template and returns the result
     *
     * @param domainModel           The domain model to transform
     * @param templateName          The name of the template that should be used to perform the transformation
     *
     * @return                      The transformed object
     */
    public String transform( DomainModel domainModel, String templateName )
    {
        return null;
    }
}
