/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.util;

import com.geekcap.modeler.model.Operation;
import com.geekcap.modeler.model.OperationParameter;

/**
 *
 * @author shaines
 */
public class OperationUtils
{
    /**
     * Returns a String containing the signature line for the specified operation.
     * 
     * For example:
     * 
     *      public List<Widget> findByPriceGreaterThan( int price )
     * 
     * @param operation             The operation for which to generate the signature
     * 
     * @return                      The signature for the operation
     */
    public static String getOperationSignature( Operation operation )
    {
        return getOperationSignature( operation, null );
    }

    /**
     * Returns a String containing the signature line for the specified operation.
     *
     * For example:
     *
     *      public List<Widget> findByPriceGreaterThan( int price )
     *
     * @param operation             The operation for which to generate the signature
     *
     * @return                      The signature for the operation
     */
    public static String getOperationSignature( Operation operation, String genericType )
    {
        StringBuilder sb = new StringBuilder();

        // Build the visibility
        if( operation.getVisibility() == Operation.Visibility.PUBLIC )
        {
            sb.append( "public " );
        }
        else if( operation.getVisibility() == Operation.Visibility.PROTECTED )
        {
            sb.append( "protected " );
        }
        else if( operation.getVisibility() == Operation.Visibility.PRIVATE )
        {
            sb.append( "private " );
        }

        // Build the return type
        sb.append( getDisplayType( operation.getReturnType(), genericType ) );
        sb.append( " " );

        // Add the operation name
        sb.append( operation.getName() );

        // Handle the operation parameters
        if( operation.getParameters().size() > 0 )
        {
            // Start the method parameters
            sb.append( "( " );

            // Add the parameters
            for( OperationParameter parameter : operation.getParameters() )
            {
                sb.append( getDisplayType( parameter.getType(), genericType ) );
                sb.append( " " );
                sb.append( parameter.getName() );
                sb.append( ", " );
            }

            // Trim off the last comma
            if( sb.charAt( sb.length() - 2 ) == ',' )
            {
                sb.delete( sb.length() - 2, sb.length() - 1 );
            }

            // Close the method parameters
            sb.append( ")" );
        }
        else
        {
            sb.append( "()" );
        }

        // Return the String that we constructed
        return sb.toString();
    }

    protected static String getDisplayType( String type, String genericType )
    {
        // Look for collections
        if( type.equalsIgnoreCase( "java.util.List" ) )
        {
            if( genericType != null )
            {
                return "List<" + genericType + ">";
            }
            return "List";
        }
        else if( type.equalsIgnoreCase( "java.util.Set" ) )
        {
            if( genericType != null )
            {
                return "Set<" + genericType + ">";
            }
            return "Set";
        }
        else if( type.equalsIgnoreCase( "java.util.Collection" ) )
        {
            if( genericType != null )
            {
                return "Collection<" + genericType + ">";
            }
            return "Collection";
        }

        // Strip off of the package
        if( type.indexOf( '.' ) != -1 )
        {
            return type.substring( type.lastIndexOf( '.' ) + 1 );
        }

        // Return the raw type
        return type;
    }
}
