/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler;

import com.geekcap.modeler.dao.ClasspathTemplateDaoImpl;
import com.geekcap.modeler.dao.PropertiesFileUmlTypeDaoImpl;
import com.geekcap.modeler.dao.Xmi14DomainObjectDaoImpl;
import com.geekcap.modeler.model.DomainModel;
import com.geekcap.modeler.model.DomainObject;
import com.geekcap.modeler.model.Field;
import com.geekcap.modeler.model.Relationship;
import com.geekcap.modeler.service.TemplateService;
import com.geekcap.modeler.service.VelocityTemplateServiceImpl;
import java.io.FileInputStream;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

/**
 * Tests the modeling functionality
 * 
 * @author shaines
 */
public class ModelerTest
{
    private TemplateService templateService;

    @Before
    public void setUp()
    {
        // Build the template service
        templateService = new VelocityTemplateServiceImpl();
        templateService.setTemplateDao( new ClasspathTemplateDaoImpl() );
    }

    @Test
    public void testHibernateTransformations()
    {
        // Create a user
        DomainObject user = new DomainObject( "User" );
        user.setPackageName( "com.geekcap.site.model" );
        user.getFields().add( new Field( "id", "long", true ) );
        user.getFields().add( new Field( "name", "String" ) );

        // Create an article
        DomainObject article = new DomainObject( "Article" );
        article.setPackageName( "com.geekcap.site.model" );
        article.getFields().add( new Field( "id", "long", true ) );
        article.getFields().add( new Field( "title", "String" ) );

        // Create a user profile object
        DomainObject profile = new DomainObject( "Profile" );
        profile.setPackageName( "com.geekcap.site.model" );
        profile.getFields().add( new Field( "id", "long", true ) );
        profile.getFields().add( new Field( "name", "String" ) );

        DomainObject role = new DomainObject( "Role" );
        role.setPackageName( "com.geekcap.site.model" );
        role.getFields().add( new Field( "id", "long", true ) );
        role.getFields().add( new Field( "name", "String" ) );

        // Define the relationships between objects
        user.getRelationships().add( new Relationship( "articles", "author", article, Relationship.Type.ONE_TO_MANY, Relationship.LoadStrategy.LAZY ) );
        article.getRelationships().add( new Relationship( "author", "articles", user, Relationship.Type.MANY_TO_ONE, Relationship.LoadStrategy.EAGER ) );
        user.getRelationships().add( new Relationship( "profile", "user", profile, Relationship.Type.ONE_TO_ONE, Relationship.LoadStrategy.EAGER, true ) );
        profile.getRelationships().add( new Relationship( "user", "profile", user, Relationship.Type.ONE_TO_ONE, Relationship.LoadStrategy.EAGER, false ) );
        user.getRelationships().add( new Relationship( "roles", "users", role, Relationship.Type.MANY_TO_MANY, Relationship.LoadStrategy.EAGER, true ) );
        role.getRelationships().add( new Relationship( "users", "roles", user, Relationship.Type.MANY_TO_MANY, Relationship.LoadStrategy.LAZY, false ) );

        // Perform the transformations
        String userHibernateClass = templateService.transform( user, "hibernate.vm" );
        String articleHibernateClass = templateService.transform( article, "hibernate.vm" );
        String profileHibernateClass = templateService.transform( profile, "hibernate.vm" );
        String roleHibernateClass = templateService.transform( role, "hibernate.vm" );

        // Validate the results
        Assert.assertNotNull( "User Hibernate class should not be null after transformation", userHibernateClass );
        Assert.assertNotNull( "Article Hibernate class should not be null after transformation", articleHibernateClass );
        Assert.assertNotNull( "Profile Hibernate class should not be null after transformation", profileHibernateClass );
        Assert.assertNotNull( "Role Hibernate class should not be null after transformation", roleHibernateClass );

        System.out.println( userHibernateClass );
        System.out.println( articleHibernateClass );
        System.out.println( profileHibernateClass );
        System.out.println( roleHibernateClass );

        Assert.assertTrue( true );
    }

    @Test
    public void testHibernateFromXmi()
    {
        try
        {
            // Open our sample XMI file
            FileInputStream fis = new FileInputStream( "src/test/resources/GeekCapSiteModel.xmi" );

            // Configure the DAO to read the domain objects
            Xmi14DomainObjectDaoImpl dao = new Xmi14DomainObjectDaoImpl();
            dao.setUmlTypeDao( new PropertiesFileUmlTypeDaoImpl() );

            // Read the domain objects
            //List<DomainObject> domainObjects = dao.loadDomainObjects( fis );
            DomainModel domainModel = dao.loadDomainModel( fis );

            // Transform the domain objects
            for( DomainObject domainObject : domainModel.getDomainObjects() )
            {
                System.out.println( "CLASS: " + domainObject.getPackageName() + "." + domainObject.getName() );
                String generatedClass = templateService.transform( domainObject, "hibernate.vm" );
                System.out.println( generatedClass );
            }

            Assert.assertTrue( true );
        }
        catch( Exception e )
        {
            e.printStackTrace();
            Assert.fail( e.getMessage() );
        }

    }
}
